/*
 * Library write_io_handle type test program
 *
 * Copyright (C) 2006-2017, Joachim Metz <joachim.metz@gmail.com>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <common.h>
#include <file_stream.h>
#include <types.h>

#if defined( HAVE_STDLIB_H ) || defined( WINAPI )
#include <stdlib.h>
#endif

#include "ewf_test_libcerror.h"
#include "ewf_test_libewf.h"
#include "ewf_test_macros.h"
#include "ewf_test_memory.h"
#include "ewf_test_unused.h"

#include "../libewf/libewf_write_io_handle.h"

#if defined( __GNUC__ )

/* Tests the libewf_write_io_handle_initialize function
 * Returns 1 if successful or 0 if not
 */
int ewf_test_write_io_handle_initialize(
     void )
{
	libcerror_error_t *error                  = NULL;
	libewf_write_io_handle_t *write_io_handle = NULL;
	int result                                = 0;

#if defined( HAVE_EWF_TEST_MEMORY )
	int number_of_malloc_fail_tests           = 1;
	int number_of_memset_fail_tests           = 1;
	int test_number                           = 0;
#endif

	/* Test regular cases
	 */
	result = libewf_write_io_handle_initialize(
	          &write_io_handle,
	          &error );

	EWF_TEST_ASSERT_EQUAL_INT(
	 "result",
	 result,
	 1 );

        EWF_TEST_ASSERT_IS_NOT_NULL(
         "write_io_handle",
         write_io_handle );

        EWF_TEST_ASSERT_IS_NULL(
         "error",
         error );

	result = libewf_write_io_handle_free(
	          &write_io_handle,
	          &error );

	EWF_TEST_ASSERT_EQUAL_INT(
	 "result",
	 result,
	 1 );

        EWF_TEST_ASSERT_IS_NULL(
         "write_io_handle",
         write_io_handle );

        EWF_TEST_ASSERT_IS_NULL(
         "error",
         error );

	/* Test error cases
	 */
	result = libewf_write_io_handle_initialize(
	          NULL,
	          &error );

	EWF_TEST_ASSERT_EQUAL_INT(
	 "result",
	 result,
	 -1 );

        EWF_TEST_ASSERT_IS_NOT_NULL(
         "error",
         error );

	libcerror_error_free(
	 &error );

	write_io_handle = (libewf_write_io_handle_t *) 0x12345678UL;

	result = libewf_write_io_handle_initialize(
	          &write_io_handle,
	          &error );

	EWF_TEST_ASSERT_EQUAL_INT(
	 "result",
	 result,
	 -1 );

        EWF_TEST_ASSERT_IS_NOT_NULL(
         "error",
         error );

	libcerror_error_free(
	 &error );

	write_io_handle = NULL;

#if defined( HAVE_EWF_TEST_MEMORY )

	for( test_number = 0;
	     test_number < number_of_malloc_fail_tests;
	     test_number++ )
	{
		/* Test libewf_write_io_handle_initialize with malloc failing
		 */
		ewf_test_malloc_attempts_before_fail = test_number;

		result = libewf_write_io_handle_initialize(
		          &write_io_handle,
		          &error );

		if( ewf_test_malloc_attempts_before_fail != -1 )
		{
			ewf_test_malloc_attempts_before_fail = -1;

			if( write_io_handle != NULL )
			{
				libewf_write_io_handle_free(
				 &write_io_handle,
				 NULL );
			}
		}
		else
		{
			EWF_TEST_ASSERT_EQUAL_INT(
			 "result",
			 result,
			 -1 );

			EWF_TEST_ASSERT_IS_NULL(
			 "write_io_handle",
			 write_io_handle );

			EWF_TEST_ASSERT_IS_NOT_NULL(
			 "error",
			 error );

			libcerror_error_free(
			 &error );
		}
	}
	for( test_number = 0;
	     test_number < number_of_memset_fail_tests;
	     test_number++ )
	{
		/* Test libewf_write_io_handle_initialize with memset failing
		 */
		ewf_test_memset_attempts_before_fail = test_number;

		result = libewf_write_io_handle_initialize(
		          &write_io_handle,
		          &error );

		if( ewf_test_memset_attempts_before_fail != -1 )
		{
			ewf_test_memset_attempts_before_fail = -1;

			if( write_io_handle != NULL )
			{
				libewf_write_io_handle_free(
				 &write_io_handle,
				 NULL );
			}
		}
		else
		{
			EWF_TEST_ASSERT_EQUAL_INT(
			 "result",
			 result,
			 -1 );

			EWF_TEST_ASSERT_IS_NULL(
			 "write_io_handle",
			 write_io_handle );

			EWF_TEST_ASSERT_IS_NOT_NULL(
			 "error",
			 error );

			libcerror_error_free(
			 &error );
		}
	}
#endif /* defined( HAVE_EWF_TEST_MEMORY ) */

	return( 1 );

on_error:
	if( error != NULL )
	{
		libcerror_error_free(
		 &error );
	}
	if( write_io_handle != NULL )
	{
		libewf_write_io_handle_free(
		 &write_io_handle,
		 NULL );
	}
	return( 0 );
}

/* Tests the libewf_write_io_handle_free function
 * Returns 1 if successful or 0 if not
 */
int ewf_test_write_io_handle_free(
     void )
{
	libcerror_error_t *error = NULL;
	int result               = 0;

	/* Test error cases
	 */
	result = libewf_write_io_handle_free(
	          NULL,
	          &error );

	EWF_TEST_ASSERT_EQUAL_INT(
	 "result",
	 result,
	 -1 );

        EWF_TEST_ASSERT_IS_NOT_NULL(
         "error",
         error );

	libcerror_error_free(
	 &error );

	return( 1 );

on_error:
	if( error != NULL )
	{
		libcerror_error_free(
		 &error );
	}
	return( 0 );
}

#endif /* defined( __GNUC__ ) */

/* The main program
 */
#if defined( HAVE_WIDE_SYSTEM_CHARACTER )
int wmain(
     int argc EWF_TEST_ATTRIBUTE_UNUSED,
     wchar_t * const argv[] EWF_TEST_ATTRIBUTE_UNUSED )
#else
int main(
     int argc EWF_TEST_ATTRIBUTE_UNUSED,
     char * const argv[] EWF_TEST_ATTRIBUTE_UNUSED )
#endif
{
	EWF_TEST_UNREFERENCED_PARAMETER( argc )
	EWF_TEST_UNREFERENCED_PARAMETER( argv )

#if defined( __GNUC__ )

	EWF_TEST_RUN(
	 "libewf_write_io_handle_initialize",
	 ewf_test_write_io_handle_initialize );

	EWF_TEST_RUN(
	 "libewf_write_io_handle_free",
	 ewf_test_write_io_handle_free );

	/* TODO: add tests for libewf_write_io_handle_clone */

	/* TODO: add tests for libewf_write_io_handle_initialize_values */

	/* TODO: add tests for libewf_write_io_handle_initialize_resume */

	/* TODO: add tests for libewf_write_io_handle_resize_table_entries */

	/* TODO: add tests for libewf_write_io_handle_calculate_chunks_per_segment_file */

	/* TODO: add tests for libewf_write_io_handle_calculate_chunks_per_section */

	/* TODO: add tests for libewf_write_io_handle_test_segment_file_full */

	/* TODO: add tests for libewf_write_io_handle_test_chunks_section_full */

	/* TODO: add tests for libewf_write_io_handle_create_segment_file */

	/* TODO: add tests for libewf_write_io_handle_write_chunks_section_start */

	/* TODO: add tests for libewf_write_io_handle_write_chunks_section_end */

	/* TODO: add tests for libewf_write_io_handle_write_new_chunk */

	/* TODO: add tests for libewf_write_io_handle_finalize_write_sections_corrections */

#endif /* defined( __GNUC__ ) */

	return( EXIT_SUCCESS );

on_error:
	return( EXIT_FAILURE );
}

